# -----------------------------------------------------------------------------------------------
# CMAKE file.
# It generates the genMake[.bat] command. 
# This utility generates the Makefile for a simulation of an application compiled by Polychrony.
# -----------------------------------------------------------------------------------------------

MK_CONFIG_NAME(${Signal_bin}/genMake SIGNAL_CMD)
GEN_HEADER_SCRIPT(${SIGNAL_CMD})
GEN_REQUIRE_SHELL_VAR(${SIGNAL_CMD} SignalLib_Utl)
GEN_REQUIRE_SHELL_VAR(${SIGNAL_CMD} SignalLibC_incl)
GEN_REQUIRE_SHELL_VAR(${SIGNAL_CMD} SignalLib_Java)
GEN_REQUIRE_SHELL_VAR(${SIGNAL_CMD} SignalLibJava_graphical)
GEN_REQUIRE_FILE(${SIGNAL_CMD} SignalLib_Utl "sigGenMakeHelp.txt")
GEN_REQUIRE_FILE(${SIGNAL_CMD} SignalLib_Utl "C_CXX_CMakeLists_template.cmake")
GEN_REQUIRE_FILE(${SIGNAL_CMD} SignalLib_Utl "Java_CMakeLists_template.cmake")
ADD_EXECUTE_RIGHT(${SIGNAL_CMD})

SET(SYNTAXDEF "ERROR: Use the command \"genMake -h\" for help")
IF(UNIX)
  FILE(APPEND ${SIGNAL_CMD} " if ( \"$#\" == 0 ) then\n")
  FILE(APPEND ${SIGNAL_CMD} "    echo \"${SYNTAXDEF}\"\n")
  FILE(APPEND ${SIGNAL_CMD} "exit\n")
  FILE(APPEND ${SIGNAL_CMD} "endif\n")
  FILE(APPEND ${SIGNAL_CMD} "\n")
  FILE(APPEND ${SIGNAL_CMD} "if ( \"$1\" == \"-h\" ) then\n")
  FILE(APPEND ${SIGNAL_CMD} "    more \${SignalLib_Utl}/sigGenMakeHelp.txt\n")
  FILE(APPEND ${SIGNAL_CMD} "exit\n")
  FILE(APPEND ${SIGNAL_CMD} "endif\n")
  FILE(APPEND ${SIGNAL_CMD} "if ( ! ( (\"$1\" == \"C\") || (\"$1\" == \"C++\") || (\"$1\" == \"Java\") || (\"$1\" == \"java\") )) then\n")
  FILE(APPEND ${SIGNAL_CMD} "    echo \"${SYNTAXDEF}. Unknown language.\"\n")
  FILE(APPEND ${SIGNAL_CMD} "exit\n")
  FILE(APPEND ${SIGNAL_CMD} "endif\n")
  FILE(APPEND ${SIGNAL_CMD} "if ( ! ( $# == 2 ) ) then\n")
  FILE(APPEND ${SIGNAL_CMD} "    echo \"${SYNTAXDEF}. Two arguments expected.\"\n")
  FILE(APPEND ${SIGNAL_CMD} "exit\n")
  FILE(APPEND ${SIGNAL_CMD} "endif\n")

  FILE(APPEND ${SIGNAL_CMD} "if (\"$1\" == \"C\") then\n")
  FILE(APPEND ${SIGNAL_CMD} "    \\cp \${SignalLib_Utl}/C_CXX_CMakeLists_template.cmake CMakeLists.txt\n")
  FILE(APPEND ${SIGNAL_CMD} "    cmake -DFOO=$2 -DLANG=C\n")
  FILE(APPEND ${SIGNAL_CMD} "    goto lab\n")
  FILE(APPEND ${SIGNAL_CMD} "endif\n")
  FILE(APPEND ${SIGNAL_CMD} "\n")
  FILE(APPEND ${SIGNAL_CMD} "if (\"$1\" == \"C++\") then\n")
  FILE(APPEND ${SIGNAL_CMD} "    \\cp \${SignalLib_Utl}/C_CXX_CMakeLists_template.cmake CMakeLists.txt\n")
  FILE(APPEND ${SIGNAL_CMD} "    cmake -DFOO=$2 -DLANG=CXX\n")
  FILE(APPEND ${SIGNAL_CMD} "    goto lab\n")
  FILE(APPEND ${SIGNAL_CMD} "endif\n")
  FILE(APPEND ${SIGNAL_CMD} "\n")
  FILE(APPEND ${SIGNAL_CMD} "\\cp \${SignalLib_Utl}/Java_CMakeLists_template.cmake CMakeLists.txt\n")
  FILE(APPEND ${SIGNAL_CMD} "cmake -DFOO=$2\n")
  FILE(APPEND ${SIGNAL_CMD} "goto lab\n")
  FILE(APPEND ${SIGNAL_CMD} "endif\n")
  FILE(APPEND ${SIGNAL_CMD} "\n")
  FILE(APPEND ${SIGNAL_CMD} "lab:\n")
  FILE(APPEND ${SIGNAL_CMD} "  echo \"To have some informations about the generated makefile, use the command: make info\"\n")

ELSE(UNIX) # windows.
  SET(XX "-G\"NMake Makefiles\"")
  FILE(APPEND ${SIGNAL_CMD} "if \"%1\" == \"\" goto errParam\n")
  FILE(APPEND ${SIGNAL_CMD} "if not \"%1%\" == \"-h\" goto testlang\n")
  FILE(APPEND ${SIGNAL_CMD} "more \"%SignalLib_Utl%\\sigGenMakeHelp.txt\"\n")
  FILE(APPEND ${SIGNAL_CMD} "goto end\n")
  FILE(APPEND ${SIGNAL_CMD} "\n")
  FILE(APPEND ${SIGNAL_CMD} ":errParam\n")
  FILE(APPEND ${SIGNAL_CMD} "echo.\n")
  FILE(APPEND ${SIGNAL_CMD} "echo ${SYNTAXDEF}\n")
  FILE(APPEND ${SIGNAL_CMD} "echo.\n")
  FILE(APPEND ${SIGNAL_CMD} "goto end\n")
  FILE(APPEND ${SIGNAL_CMD} "\n")
  FILE(APPEND ${SIGNAL_CMD} ":testlang\n")
  FILE(APPEND ${SIGNAL_CMD} "if \"%1%\" == \"C\" goto setoklangC\n")
  FILE(APPEND ${SIGNAL_CMD} "if \"%1%\" == \"C++\" goto setoklangCXX\n")
  FILE(APPEND ${SIGNAL_CMD} "if \"%1%\" == \"Java\" goto setoklangJava\n")
  FILE(APPEND ${SIGNAL_CMD} "if \"%1%\" == \"java\" goto setoklangJava\n")
  FILE(APPEND ${SIGNAL_CMD} "echo.\n")
  FILE(APPEND ${SIGNAL_CMD} "echo ${SYNTAXDEF}\n")
  FILE(APPEND ${SIGNAL_CMD} "echo.\n")
  FILE(APPEND ${SIGNAL_CMD} "goto end\n")
  FILE(APPEND ${SIGNAL_CMD} "\n")

  FILE(APPEND ${SIGNAL_CMD} ":setoklangC\n")
  FILE(APPEND ${SIGNAL_CMD} "copy \"%SignalLib_Utl%\\C_CXX_CMakeLists_template.cmake\" CMakeLists.txt\n")
  FILE(APPEND ${SIGNAL_CMD} "cmake -DFOO=%2% -DLANG=C ${XX}\n")
  FILE(APPEND ${SIGNAL_CMD} "echo.\n")
  FILE(APPEND ${SIGNAL_CMD} "goto end1\n")

  FILE(APPEND ${SIGNAL_CMD} ":setoklangCXX\n")
  FILE(APPEND ${SIGNAL_CMD} "copy \"%SignalLib_Utl%\\C_CXX_CMakeLists_template.cmake\" CMakeLists.txt\n")
  FILE(APPEND ${SIGNAL_CMD} "cmake -DFOO=%2% -DLANG=CXX ${XX}\n")
  FILE(APPEND ${SIGNAL_CMD} "echo.\n")
  FILE(APPEND ${SIGNAL_CMD} "goto end1\n")

  FILE(APPEND ${SIGNAL_CMD} ":setoklangJava\n")
  FILE(APPEND ${SIGNAL_CMD} "copy \"%SignalLib_Utl%\\Java_CMakeLists_template.cmake\" CMakeLists.txt\n")
  FILE(APPEND ${SIGNAL_CMD} "cmake -DFOO=%2% \n")
  FILE(APPEND ${SIGNAL_CMD} "echo.\n")

  FILE(APPEND ${SIGNAL_CMD} ":end1\n")
  FILE(APPEND ${SIGNAL_CMD} "echo \"To have some informations about the generated makefile, use the command: nmake info\"\n")

  FILE(APPEND ${SIGNAL_CMD} ":end\n")
ENDIF(UNIX)

