#include "TYPES.h"
#include "BlackboardManager.h"


/*****************************************   BLACKBOARD OBJECT ****************************************/


Blackboard::Blackboard (APEX_Blackboard_type b):board (b)
{
}




/************************************** BLACKBOARD MANAGER_NODE  **************************************/

BlackboardManagerNode::BlackboardManagerNode (BoardAssoc_type value):next (NULL)
{
  contents.BlackboardID = value.BlackboardID;
  contents.ptrBlackboard = value.ptrBlackboard;
}

BlackboardManagerNode::BlackboardManagerNode ()
{
}

BlackboardManagerNode::~BlackboardManagerNode ()
{
}

void
BlackboardManagerNode::setValue (BoardAssoc_type value)
{
  contents.BlackboardID = value.BlackboardID;
  contents.ptrBlackboard = value.ptrBlackboard;
}

void
BlackboardManagerNode::setNext (ptBlackboardManagerNode ptvalue)
{
  next = ptvalue;
}

BoardAssoc_type
BlackboardManagerNode::getValue ()
{
  return contents;
}

ptBlackboardManagerNode
BlackboardManagerNode::getNext ()
{
  return next;
}



/*************************************** BLACKBOARD MANAGER  *****************************************/

BlackboardManager::BlackboardManager (int size):first (NULL), last (NULL), max_size (size), current_size (0)
{
}

BlackboardManager::BlackboardManager ():first (NULL), last (NULL), max_size (MAX_NUMBER_OF_BLACKBOARDS), current_size (0)
{
}

BlackboardManager::~BlackboardManager ()
{
}

void
BlackboardManager::setFirst (ptBlackboardManagerNode ptvalue)
{
  first = ptvalue;
}

void
BlackboardManager::setLast (ptBlackboardManagerNode ptvalue)
{
  last = ptvalue;
}

ptBlackboardManagerNode
BlackboardManager::getFirst ()
{
  return first;
}

ptBlackboardManagerNode
BlackboardManager::getLast ()
{
  return last;
}

bool
BlackboardManager::BlackboardRecord (Comm_ComponentName_type blackboard_Name, int message_Size, Comm_ComponentID_type & blackboardID)
{
  if (current_size < max_size)
    {
      current_size++;
      BoardAssoc_type new_record;
      new_record.BlackboardID = current_size;

      APEX_Blackboard_type board;
      strcpy (board.Blackboard_Name, blackboard_Name);
      board.Empty_Indicator = pk_true;
      board.Waiting_Processes = 0;
      board.Message_Size = message_Size;
      for (int i = 0; i < MAX_NUMBER_OF_PROCESSES; i++)
	{
	  board.Waiting_Queue[i] = pk_false;
	};

      new_record.ptrBlackboard = new Blackboard (board);
      blackboardID = current_size;

      ptBlackboardManagerNode new_node = new BlackboardManagerNode (new_record);

      if (first == NULL)
	{
	  first = new_node;
	  last = new_node;
	}
      else
	{
	  ptBlackboardManagerNode iter = first;
	  while (iter != last)
	    {
	      iter = iter->getNext ();
	    };
	  if (iter == last)
	    {
	      last->setNext (new_node);
	      last = new_node;
	    }
	};
      return pk_true;
    }
  else
    {
      return pk_false;
    };
}


bool
BlackboardManager::BlackboardCheckName (Comm_ComponentName_type blackboardName, Comm_ComponentID_type & blackboardID)
{
  bool retour;
  if (first == NULL)
    {
      blackboardID = NULL;
      retour = pk_false;
    }
  else
    {
      ptBlackboardManagerNode iter = first;
      while (strcmp ((((iter->getValue ()).ptrBlackboard)->board).Blackboard_Name, blackboardName) && (iter != last))
	{
	  iter = iter->getNext ();
	};
      if (iter == last)
	{
	  if (strcmp ((((iter->getValue ()).ptrBlackboard)->board).Blackboard_Name, blackboardName))
	    {
	      blackboardID = NULL;
	      retour = pk_false;
	    }
	  else
	    {
	      blackboardID = (iter->getValue ()).BlackboardID;
	      retour = pk_true;
	    }
	}
      else
	{
	  blackboardID = (iter->getValue ()).BlackboardID;
	  retour = pk_true;
	}
    };
  return retour;
}


bool
BlackboardManager::BlackboardCheckID (Comm_ComponentID_type blackboardID, APEX_Blackboard_type & board)
{
  bool retour;
  if (first == NULL)
    {
      retour = pk_false;
    }
  else
    {
      ptBlackboardManagerNode iter = first;
      while (((iter->getValue ()).BlackboardID != blackboardID) && (iter != last))
	{
	  iter = iter->getNext ();
	};
      if (iter == last)
	{
	  if ((iter->getValue ()).BlackboardID != blackboardID)
	    {
	      retour = pk_false;
	    }
	  else
	    {
	      board = ((last->getValue ()).ptrBlackboard)->board;
	      retour = pk_true;
	    }
	}
      else
	{
	  board = ((iter->getValue ()).ptrBlackboard)->board;
	  retour = pk_true;
	}
    };
  return retour;
}


void
BlackboardManager::BlackboardUpdate (APEX_Blackboard_type board)
{
  bool b;
  APEX_Blackboard_type *x;

  if (first != NULL)
    {
      ptBlackboardManagerNode iter = first;
      Comm_ComponentName_type name;
      strcpy (name, board.Blackboard_Name);
      while (strcmp ((((iter->getValue ()).ptrBlackboard)->board).Blackboard_Name, name) && (iter != last))
	iter = iter->getNext ();

      if (iter != last)
	b = pk_true;
      else
	b = !strcmp ((((iter->getValue ()).ptrBlackboard)->board).Blackboard_Name, name);
      if (b)
	{
	  x = &(((iter->getValue ()).ptrBlackboard)->board);
	  x->Message = board.Message;
	  x->Empty_Indicator = board.Empty_Indicator;
	  x->Waiting_Processes = board.Waiting_Processes;
	  for (int i = 0; i < MAX_NUMBER_OF_PROCESSES; i++)
	    x->Waiting_Queue[i] = board.Waiting_Queue[i];
	}
    }
}

bool
BlackboardManager::Full ()
{
  return (max_size == current_size);
}

void
BlackboardManager::printFile (Comm_ComponentID_type blackboardID, char *result_file)
{
  FILE *file_out;
  APEX_Blackboard_type board;
  if (this->BlackboardCheckID (blackboardID, board))
    {
      file_out = fopen (result_file, "wt");
      if (!file_out)
	{
	  fprintf (stderr, "Can not open file %s\n", result_file);
	  exit (1);
	};
      fprintf (file_out, "(Blackboard_Identifier : %i, Name : %s, Displayed_Message : %i, Empty_Indicator : %i, Waiting_Processes : %i) \n",
	       blackboardID, board.Blackboard_Name, board.Message, board.Empty_Indicator, board.Waiting_Processes);
      fclose (file_out);
    }
}
