#include "ProcessDescriptor.h"




			      /*============================*/
			      /* Author: A. GAMATIE         */
			      /* Last modified 03/13/2003   */
			      /*============================*/

/**************************************************************************************/
/****************************  class PROCESS DESCRIPTOR (body) ************************/
/**************************************************************************************/


ProcessDescriptor::ProcessDescriptor (ProcessName_type P_name,
				      SystemAddress_type P_entrypoint,
				      StackSize_type P_stacksize,
				      Priority_type P_basepriority,
				      SystemTime_type P_period, SystemTime_type P_timecapacity, Deadline_type P_deadline)
{
  strcpy ((ProcessStatus.Attributes).Name, P_name);
  (ProcessStatus.Attributes).Entry_Point = P_entrypoint;
  (ProcessStatus.Attributes).Stack_Size = P_stacksize;
  (ProcessStatus.Attributes).Base_Priority = P_basepriority;
  (ProcessStatus.Attributes).Period = P_period;
  (ProcessStatus.Attributes).Time_Capacity = P_timecapacity;
  (ProcessStatus.Attributes).Deadline = P_deadline;
  ProcessStatus.Current_Priority = P_basepriority;
  ProcessStatus.Deadline_Time = P_timecapacity;
  ProcessStatus.Process_State = DORMANT;
  WaitIndicator = 0;
  for (int i = 0; i < MAX_RESOURCES; i += 1)
    ResourceUsed[i] = -1;
  CreationIndicator = 0;
  SuspensionIndicator = 0;
}


ProcessDescriptor::ProcessDescriptor ()
{
  ProcessStatus.Current_Priority = -1;
  WaitIndicator = 0;
  for (int i = 0; i < MAX_RESOURCES; i += 1)
    ResourceUsed[i] = -1;
  CreationIndicator = 0;
  SuspensionIndicator = 0;
  WaitingTime = 0.0;
}


ProcessDescriptor::ProcessDescriptor (ProcessDescriptor & desc)
{
  WaitIndicator = desc.WaitIndicator;
  for (int i = 0; i < MAX_RESOURCES; i += 1)
    ResourceUsed[i] = (desc.ResourceUsed)[i];
  ProcessStatus = desc.ProcessStatus;
  CreationIndicator = desc.CreationIndicator;
  SuspensionIndicator = desc.SuspensionIndicator;
  WaitingTime = desc.WaitingTime;
  Message = desc.Message;
}


ProcessDescriptor::~ProcessDescriptor ()
{
}


void
ProcessDescriptor::setProcessName (ProcessName_type P_name)
{
  strcpy ((ProcessStatus.Attributes).Name, P_name);
}


void
ProcessDescriptor::getProcessName (ProcessName_type & P_name)
{
  strcpy (P_name, (ProcessStatus.Attributes).Name);
}


void
ProcessDescriptor::setEntryPoint (SystemAddress_type P_entryPt)
{
  (ProcessStatus.Attributes).Entry_Point = P_entryPt;
}


SystemAddress_type
ProcessDescriptor::getEntryPoint ()
{
  return (ProcessStatus.Attributes).Entry_Point;
}


void
ProcessDescriptor::setStackSize (StackSize_type P_stacksize)
{
  (ProcessStatus.Attributes).Stack_Size = P_stacksize;
}


StackSize_type
ProcessDescriptor::getStackSize ()
{
  return (ProcessStatus.Attributes).Stack_Size;
}


void
ProcessDescriptor::setBasePriority (Priority_type P_basepriority)
{
  (ProcessStatus.Attributes).Base_Priority = P_basepriority;
}


Priority_type
ProcessDescriptor::getBasePriority ()
{
  return (ProcessStatus.Attributes).Base_Priority;
}


void
ProcessDescriptor::setPeriod (SystemTime_type P_period)
{
  (ProcessStatus.Attributes).Period = P_period;
}


SystemTime_type
ProcessDescriptor::getPeriod ()
{
  return (ProcessStatus.Attributes).Period;
}


void
ProcessDescriptor::setTimeCapacity (SystemTime_type P_timecap)
{
  (ProcessStatus.Attributes).Time_Capacity = P_timecap;
}


SystemTime_type
ProcessDescriptor::getTimeCapacity ()
{
  return (ProcessStatus.Attributes).Time_Capacity;
}


void
ProcessDescriptor::setDeadline (Deadline_type P_deadline)
{
  (ProcessStatus.Attributes).Deadline = P_deadline;
}


Deadline_type
ProcessDescriptor::getDeadline ()
{
  return (ProcessStatus.Attributes).Deadline;
}


void
ProcessDescriptor::setCurrentPriority (Priority_type P_currentpriority)
{
  ProcessStatus.Current_Priority = P_currentpriority;
}


Priority_type
ProcessDescriptor::getCurrentPriority ()
{
  return ProcessStatus.Current_Priority;
}


void
ProcessDescriptor::setDeadlineTime (SystemTime_type P_dltime)
{
  ProcessStatus.Deadline_Time = P_dltime;
}


SystemTime_type
ProcessDescriptor::getDeadlineTime ()
{
  return ProcessStatus.Deadline_Time;
}


void
ProcessDescriptor::setProcessState (ProcessState_type P_state)
{
  ProcessStatus.Process_State = P_state;
}


ProcessState_type
ProcessDescriptor::getProcessState ()
{
  return ProcessStatus.Process_State;
}


void
ProcessDescriptor::setWaitIndicator (logical P_waitind)
{
  WaitIndicator = P_waitind;
}


logical
ProcessDescriptor::getWaitIndicator ()
{
  return WaitIndicator;
}


void
ProcessDescriptor::setProcessAttributes (ProcessAttributes_type Attributes)
{
  ProcessStatus.Attributes = Attributes;
}


ProcessAttributes_type
ProcessDescriptor::getProcessAttributes ()
{
  return ProcessStatus.Attributes;
}


void
ProcessDescriptor::setProcessStatus (ProcessStatus_type Status)
{
  ProcessStatus = Status;
}


ProcessStatus_type
ProcessDescriptor::getProcessStatus ()
{
  return ProcessStatus;
}


void
ProcessDescriptor::setCreationIndicator (logical created)
{
  CreationIndicator = created;
}


logical
ProcessDescriptor::getCreationIndicator ()
{
  return CreationIndicator;
}


void
ProcessDescriptor::setWaitingTime (SystemTime_type delay)
{
  WaitingTime = delay;
}


SystemTime_type
ProcessDescriptor::getWaitingTime ()
{
  return WaitingTime;
}


void
ProcessDescriptor::incrementWaitingTime ()
{
  WaitingTime++;
}


void
ProcessDescriptor::incrementWaitingTimeProfiling (double val)
{
  WaitingTime = WaitingTime + val;
}


void
ProcessDescriptor::decrementWaitingTime ()
{
  if (WaitingTime > 0)
    WaitingTime--;
}


void
ProcessDescriptor::decrementWaitingTimeProfiling (double val)
{
  if (WaitingTime > 0)
    WaitingTime = WaitingTime - val;
}


logical
ProcessDescriptor::nullWaitingTime ()
{
  return (WaitingTime <= 0.0);
}


void
ProcessDescriptor::insertResource (Comm_ComponentID_type component)
{
  int i = 0;
  while ((ResourceUsed[i] != -1) && (i < MAX_RESOURCES))
    i += 1;
  if ((ResourceUsed[i] == -1) && (i < MAX_RESOURCES))
    ResourceUsed[i] = component;
}


void
ProcessDescriptor::removeResource (Comm_ComponentID_type component)
{
  int i = 0;
  while ((ResourceUsed[i] != component) && (i < MAX_RESOURCES))
    i += 1;
  if ((ResourceUsed[i] == component) && (i < MAX_RESOURCES))
    ResourceUsed[i] = -1;
}


void
ProcessDescriptor::removAllResource ()
{
  for (int i = 0; i < MAX_RESOURCES; i += 1)
    ResourceUsed[i] = -1;
}


Message_type
ProcessDescriptor::getMessage ()
{
  return Message;
}


void
ProcessDescriptor::setMessage (Message_type mess)
{
  Message = mess;
}


logical
ProcessDescriptor::getSuspensionIndicator ()
{
  return SuspensionIndicator;
}


void
ProcessDescriptor::setSuspensionIndicator (logical suspension)
{
  SuspensionIndicator = suspension;
}
